# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/24 0:21
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : __init__.py.py
# @Software: PyCharm
from . import calculate_sat, sat_handle




# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/21 2:09
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : __init__.py.py
# @Software: PyCharm
from . import (
    exercise,
    mgsl,
    sat,
    log,
    path,
    utils
               )



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/21 2:28
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : calculate_sat.py
# @Software: PyCharm
import json
import re
from typing import Dict

from typing import Optional

from ..log import log
from ..utils import *
from ..path import *
import ephem

logger = log()


async def get_tian_gong():
    Path.mkdir(LOCAL) if not Path.exists(LOCAL) else ...
    logger.debug("读取在线数据...")
    tian_gong_ = await http_get('http://www.cmse.gov.cn/gfgg/zgkjzgdcs/')
    selected = re.findall(re.compile(r'<div><font face="Courier New">(.*?)</font></div>'), tian_gong_)
    line1 = selected[0].replace('&nbsp;', ' ')
    line2 = selected[1].replace('&nbsp;', ' ')
    tg_data = {"天宫": [line1, line2]}
    logger.debug("写入本地数据...")
    await write_(TIANGONG, json.dumps(str(tg_data)))
    return tg_data


async def download_ham_sat():
    Path.mkdir(LOCAL) if not Path.exists(LOCAL) else ...
    await get_tian_gong()
    logger.debug("读取在线数据...")
    url = "https://amsat.org/tle/current/nasabare.txt"
    ham_sat_ = await http_get(url)
    if not ham_sat_:
        logger.error(f"从 {url} 下载数据出错")
        return None
    else:
        logger.debug("写入本地数据...")
        await write_(HAM_SAT, str(ham_sat_))
        return ham_sat_


async def data2Tle() -> Dict[str, list]:
    # if not Path.exists(HAM_SAT):
    # FIXME 每次都下载数据，计划进行缓存，根据上次下载的时间来判断是否需要下载数据
    await download_ham_sat()
    content = (await read_all(HAM_SAT))
    logger.info("载入数据...")
    count = 0
    data = {}
    temp = []
    # 将数据分割成行TLE轨道报
    for line in content.split('\n'):
        count += 1
        temp.append(line)
        if count % 3 == 0:
            data[temp[0]] = temp[1:]
            count = 0
            temp = []
    await write_(DATA_DICT, json.dumps(str(data)))
    return data


async def calculate(name: str, location: list, time=ephem.now()) -> Optional[list]:
    """

    :param name: 卫星名
    :param location: QTH[经度，纬度，海拔]
    :param time: 时间
    :return: [方位角，仰角，相对速率]
    """
    logger.debug(f"触发计算：{str(time)}")
    logger.info(f"计算卫星:{name}")
    logger.debug("QTJ:{str(location)}")

    me = ephem.Observer()
    me.lon, me.lat, me.elevation = location[0], location[1], float(location[2])
    if isChinese(name) and name in ['天宫', '天宫号', '中国空间站']:
        tle = (await get_tian_gong())
        name = "天宫"
    elif name.upper() == "TIANGONG":
        tle = (await get_tian_gong())
        name = "天宫"
    else:
        tle = (await data2Tle())
    try:
        sat = ephem.readtle(name, tle[name][0], tle[name][1])
        me.date = time
        sat.compute(me)
        az = str(sat.az * 180.0 / 3.1415926535)
        alt = str(sat.alt * 180.0 / 3.1415926535)
        range_velocity = str(sat.range_velocity)
        logger.debug("方位角： " + az)  # 卫星的方位角
        logger.debug("仰角： " + alt)  # 卫星的仰角
        logger.debug("运动速率： " + range_velocity)  # 卫星相对于观察者的运动速率，为正，表示卫星正在远离观察者。
        return [az, alt, range_velocity]
    except KeyError:
        logger.error("未找到卫星：" + name)
        return None


if __name__ == '__main__':
    import asyncio

    loop = asyncio.get_event_loop()
    loop.run_until_complete(calculate("PO-101"))



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/21 2:20
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : path.py
# @Software: PyCharm
from pathlib import Path

from nonebot import get_driver
from nonebot import logger


#  新建一个类，用来存放各种路径

LOCAL = Path() / 'cqsat'
HAM_SAT = LOCAL / 'ham_sat.txt'
TIANGONG = LOCAL / 'tiangong.json'
CONFIG = LOCAL / 'config.yml'
TEMP = LOCAL / 'temp.yaml'
DATA_DICT = LOCAL / 'data.json'
QTH = LOCAL / 'qth.yml'
LOG = LOCAL / 'log.log'
EXERCISE_TEMP = LOCAL / 'exercise_temp.yml'
EXAM_CACHE = LOCAL / 'exam'
EXAM_CACHE_A = EXAM_CACHE / 'A'
EXAM_CACHE_B = EXAM_CACHE / 'B'
EXAM_CACHE_C = EXAM_CACHE / 'C'

path_list = [LOCAL, EXAM_CACHE, EXAM_CACHE_A, EXAM_CACHE_B, EXAM_CACHE_C]
driver = get_driver()
global_config = driver.config


@driver.on_bot_connect
async def _():
    await plugin_init()


async def plugin_init():
    """
    初始化插件
    :return:
    """
    for dir_ in path_list:
        if not dir_.exists():
            dir_.mkdir()
    logger.success('cqsat插件初始化检测完成')



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/23 22:36
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : calculate_mgsl.py
# @Software: PyCharm
import math

index2Char = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U',
              'V', 'W', 'X', 'Y', 'Z']


# 计算场编号
async def getField(long: float, latitude: float):
    global index2Char
    field1 = math.floor((180 + long) / 20)
    field2 = math.floor((90 + latitude) / 10)
    # print('场计算：%s-%s' % (field1,field2))
    field = index2Char[field1] + index2Char[field2]
    # print('场代码：%s' % field)
    return field


# 计算方编号
async def getSquare(long: float, latitude: float):
    square1 = math.floor(math.floor(long + 180) % 20 / 2)
    square2 = math.floor(latitude + 90) % 10
    # print('方计算：%s-%s' % (square1,square2))
    square = str(square1) + str(square2)
    # print('方代码：%s' % square)
    return square


# 计算块编号
async def getSubSquare(long: float, latitude: float):
    global index2Char
    subSquare1 = math.floor((long - math.floor(long / 2) * 2) * 60 / 5)
    subSquare2 = math.floor((latitude - math.floor(latitude)) * 60 / 2.5)
    # print('块计算:%s-%s' % (subSquare1,subSquare2))
    subSquare = index2Char[subSquare1] + index2Char[subSquare2]
    # print('块代码：%s' % subSquare)
    return subSquare


async def getGrid(long: float, latitude: float):
    gridPos = await getField(long, latitude)
    gridPos += await getSquare(long, latitude)
    gridPos += await getSubSquare(long, latitude)
    return gridPos


if __name__ == '__main__':
    lon = 75.8656
    lat = 39.3809
    a = getGrid(lon, lat)
    print(a)



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/31 3:11
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : pre.py
# @Software: PyCharm
import json
from pathlib import Path

RESOURCE = Path.cwd().parent / "cqsat_resource"
BANK = RESOURCE / "bank"


def txt2json(level: str) -> None:
    """
    将题库txt文件转换为json文件
    :param level: 难度
    :return:
    """
    with open(f"{level}.txt", 'r', encoding='utf-8') as f:
        data = f.read()
    data = data.split('\n')
    data = list(filter(None, data))
    count = 0
    temp = []
    sum_ = []
    dic = {}
    for i in range(len(data)):
        print(count)
        if count == 6:
            temp.append(data[i])
            count = 0
            sum_.append(temp)
            temp = []
        else:
            temp.append(data[i])
            count += 1
    s2 = []
    for i in range(len(sum_)):
        question = sum_[i]
        temp = []
        for j in question:
            j = j.replace("[", "")
            j = j.split("]")
            temp.append(j)
        s2.append(temp)
    for i in range(len(s2)):
        temp_dic = {}
        for j in s2[i]:
            count2 = str(i + 1)
            temp_dic[j[0]] = j[1]
            dic[count2] = temp_dic
    with open(BANK / f"{level}.json", 'w', encoding='utf-8') as f:
        json.dump(dic, f, ensure_ascii=False, indent=4)


if __name__ == '__main__':
    txt2json("A")
    txt2json("B")
    txt2json("C")



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/9/5 3:42
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : exam.py
# @Software: PyCharm
import datetime
import json
import random

from nonebot import on_command, require, Bot
from nonebot.adapters.onebot.v11 import MessageSegment, MessageEvent, Message, GroupMessageEvent, PrivateMessageEvent, \
    Event
from nonebot.internal.params import ArgStr, ArgPlainText
from nonebot.matcher import Matcher
from nonebot.params import CommandArg
from nonebot.typing import T_State

from ..path import *
from ..utils import *

RESOURCE = Path(__file__).parent.parent / "cqsat_resource"
BANK = RESOURCE / "bank"
IMG = BANK / 'imgs'
do_exam = on_command("ham考试", aliases={"Ham考试", "HAM考试", "H考试", "h考试"}, block=True)


@do_exam.got("level", prompt="请你要进行哪个等级的考试？（A B C）")
async def _(
        matcher: Matcher,
        event: Event,
        state: T_State,
        answer=None,
        level: str = ArgStr("level")):
    global USER_PATH
    if isinstance(event, GroupMessageEvent):
        qq = event.user_id
        state["user_notice"] = MessageSegment.at(qq)
    elif isinstance(event, PrivateMessageEvent):
        qq = event.user_id
        state["user_notice"] = ""
    if level in ["取消", "算了", "不考了", "退出考试", "退出"]:
        await do_exam.finish("已取消操作...")
    elif level.upper() not in ["A", "B", "C"]:
        await do_exam.reject("输入错误，请重新输入...")
    elif level.upper() in ["A", "B", "C"]:
        USER_PATH = EXAM_CACHE / level.upper() / f"{qq}.yml"
        if not Path.exists(USER_PATH):
            await yaml_dump(USER_PATH, {"done": 0, "best": 0, "wrong": {}, "Accuracy": []})
        state['level'] = level.upper()
        state['qq'] = qq
    user_history = (await yaml_load(USER_PATH))
    already_done = user_history["done"]
    if already_done != 0:
        best = user_history["best"]
        wrong = user_history["wrong"]
        wrong_to_send = ""
        if wrong:
            for i in wrong:
                wrong_to_send += f"{i}: {wrong[i]}次\n"
        else:
            wrong_to_send = "无"
        await do_exam.send(state["user_notice"] + f"你在此前已经完成过{already_done}次考试，最高分为{best}分，错题情况如下：\n{wrong_to_send}")

    state['last'] = 0
    state["correct"] = 0
    state["wrong"] = {}
    state["ex_list"] = None
    state["time_is_up"] = None
    state["is_send"] = False


@do_exam.got("answer", prompt="回复任意内容开始...")
async def _(matcher: Matcher, state: T_State, reply: str = ArgPlainText("answer")):
    global ex_list
    level = state["level"]
    qq = state["qq"]
    USER_PATH = EXAM_CACHE / level.upper() / f"{qq}.yml"
    last = state['last']
    exam_config_dict = {
        "A": [30, 30, 25],
        "B": [50, 60, 40],
        "C": [80, 90, 60]
    }
    this_config = exam_config_dict[level]
    state["this_config"] = this_config
    if not state["time_is_up"]:
        time_up = datetime.datetime.now() + datetime.timedelta(minutes=this_config[1])
        await do_exam.send(f"考试于{time_up.strftime('%H:%M:%S')}结束")
        state["time_is_up"] = time_up
    #     在time_up这个时间设置定时任务，如果到了这个时间，就执行finish_matcher_cause_time
        scheduler = require("nonebot_plugin_apscheduler").scheduler
        scheduler.add_job(
            finish_matcher_cause_time,
            "date",
            run_date=time_up,
            args=[matcher]
        )

    ex_bank = json.loads(await read_all(BANK / f"{level}.json"))
    if not state["ex_list"]:
        ex_list = random.sample(range(1, len(ex_bank)), this_config[0])
        state["ex_list"] = ex_list
    else:
        pass
    if last == 0:
        last = ex_list[0]
        state['last'] = last
    if reply in ["取消", "算了", "退出", "不做了", "怎么退出"]:
        await do_exam.finish("已取消操作...")
    if reply in ["时间", "还有多久结束", "还剩多久结束", "查看时间"]:
        time_is_up = state["time_is_up"]
        time_left = time_is_up - datetime.datetime.now()
        time_left = time_left.seconds // 60
        await do_exam.send(state["user_notice"] + f"距离考试结束还有{time_left}分钟")

    if state["is_send"]:
        if reply.upper() in ["A", "B", "C", "D"]:
            if ex_list.index(int(last)) < len(ex_list) - 1:
                for i in range(this_config[0]):
                    # if state["this_send"][reply.upper()] != state["this_answer"]:
                    if reply.upper() != state["this_answer"]:
                        await do_exam.send(f"× 应为 {state['this_answer']}"+ "\n"+ state["user_notice"])
                        last_ = last
                        last = ex_list[ex_list.index(last) + 1]
                        state["last"] = last
                        wrong = state["wrong"]
                        wrong[last_] = 0
                        wrong[last_] += 1
                        state["wrong"] = wrong
                        state = (
                            await send_ex(
                                matcher,
                                ex_bank,
                                last,
                                qq,
                                level,
                                state,
                                img=IMG,
                                mode="exam"
                            )
                        )
                    elif reply.upper() == state["this_answer"]:
                        last = ex_list[ex_list.index(last) + 1]
                        state["last"] = last
                        await do_exam.send("√")
                        state["correct"] += 1
                        state = (
                            await send_ex(
                                matcher,
                                ex_bank,
                                last,
                                qq,
                                level,
                                state,
                                img=IMG,
                                mode="exam"
                            )
                        )
            else:
                if reply.upper() == state["this_answer"]:
                    await do_exam.send("√")
                    state["correct"] += 1
                elif reply.upper() != state["this_answer"]:
                    await do_exam.send(f"× 应为 {state['this_answer']}")
                    wrong = state["wrong"]
                    wrong[last] = 0
                    wrong[last] += 1
                    state["wrong"] = wrong

                user_history = await yaml_load(USER_PATH)
                user_history["done"] += 1
                user_history["best"] = max(user_history["best"], state["correct"])
                for i in user_history["wrong"]:
                    if i in state["wrong"]:
                        user_history["wrong"][i] += state["wrong"][i]
                        state["wrong"].pop(i)
                user_history["wrong"].update(state["wrong"])
                logger.info(user_history)
                user_history["Accuracy"].append(state["correct"] / this_config[0])
                await yaml_dump(USER_PATH, user_history)
                await do_exam.send(state["user_notice"] + "已完成本次考试")
                await do_exam.send(state["user_notice"] + "\n" +
                                   f"本次考试共【{this_config[0]}】题\n"
                                   f"正确【{state['correct']}】题\n"
                                   f"错误【{len(state['wrong'])}】题\n"
                                   f"{'-' * 10}\n"
                                   f"{'祝贺你考试通过！' if state['correct'] >= this_config[2] else '很遗憾本次考试未通过...再接再厉！'}\n"
                                   f"{'-' * 10}\n"
                                   f"正确率为【{round(state['correct'] / this_config[0] * 100, 2)}%】\n"
                                   f"你 {level.upper()} 类题目的平均正确率为【{round(sum(user_history['Accuracy']) / len(user_history['Accuracy']), 2) * 100}%】 "
                                   )
                cache_path = EXAM_CACHE / state["level"].upper() / f"{state['qq']}_state.txt"
                if cache_path.exists():
                    cache_path.unlink()
                else:
                    logger.warning(f"cache文件不存在，无法删除")

        else:
            await do_exam.reject("输入错误，请重新输入上方题目的答案...")
    elif not state["is_send"]:
        state["is_send"] = True
        state = (
            await send_ex(
                matcher,
                ex_bank,
                last,
                qq,
                level,
                state,
                img=IMG,
                mode="exam"
            )
        )


async def finish_matcher_cause_time(matcher: Matcher):
    """
    停止考试
    :param matcher: Matcher
    :return:
    """
    cache_path = EXAM_CACHE / matcher.state["level"].upper() / f"{matcher.state['qq']}_state.txt"
    state = (await read_all(cache_path))
    state = json.loads(state)
    await matcher.send("时间到，考试结束！")
    if state["correct"] + len(state["wrong"]) != state['this_config'][0]:
        await matcher.send(f"你还有【{state['this_config'][0] - state['correct'] - len(state['wrong'])}】题未做")

    await matcher.send(
        f"本次考试共【{state['this_config'][0]}】题\n"
        f"正确【{state['correct']}】题\n"
        f"错误【{len(state['wrong'])}】题\n"
        f"{'-' * 10}\n"
        f"{'祝贺你考试通过！' if state['correct'] >= state['this_config'][2] else '很遗憾本次考试未通过...再接再厉！'}\n"
        f"{'-' * 10}\n"
    )
#      删除cache
    if cache_path.exists():
        cache_path.unlink()
    else:
        logger.warning(f"cache文件不存在，无法删除")


my_wrong = on_command("我的错题", aliases={"我的错题本"}, priority=5, block=True)


@my_wrong.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    u_path = f"{event.user_id}.yml"
    wrong_a = (await yaml_load(EXAM_CACHE_A / u_path))["wrong"] if (EXAM_CACHE_A / u_path).exists() else {}
    wrong_b = (await yaml_load(EXAM_CACHE_B / u_path))["wrong"] if (EXAM_CACHE_B / u_path).exists() else {}
    wrong_c = (await yaml_load(EXAM_CACHE_C / u_path))["wrong"] if (EXAM_CACHE_C / u_path).exists() else {}
    r_a = ""
    if wrong_a:
        for i in wrong_a:
            r_a += f"{i}: {wrong_a[i]}次\n"
    r_b = ""
    if wrong_b:
        for i in wrong_b:
            r_b += f"{i}: {wrong_b[i]}次\n"
    r_c = ""
    if wrong_c:
        for i in wrong_c:
            r_c += f"{i}: {wrong_c[i]}次\n"
    await my_wrong.send(
        f"你的错题有【{len(wrong_a) + len(wrong_b) + len(wrong_c)}】道题目\n"
        f"{'-' * 10}\n"
        f"A类：{r_a if r_a else 0}\n"
        f"{'-' * 10}\n"
        f"B类：{r_b if r_b else 0}\n"
        f"{'-' * 10}\n"
        f"C类：{r_c if r_c else 0}\n"
                        )


refer_question = on_command("查题", aliases={"查询题目"}, priority=5, block=True)

@refer_question.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State, args:Message=CommandArg()):
    if not args:
        await refer_question.finish("请发送【查题 级别 题号】\n例如：\n  查题 A 1\n级别和题号之间用空格分割")
    else:
        a_list = str(args).split(" ")
        if len(a_list) != 2:
            await refer_question.finish("请发送【查题 级别 题号】\n例如：\n  查题 A 1\n级别和题号之间用空格分割")
        else:
            for i in a_list:
                if i.isdigit():
                    num = i
                    a_list.pop(a_list.index(i))
                    level = a_list[0].upper()
                else:
                    await refer_question.finish("请发送【查题 级别 题号】\n例如：\n  查题 A 1\n级别和题号之间用空格分割")
            if level not in ["A", "B", "C"]:
                await refer_question.finish("级别错误，应该为A、B、C其中一个")
            else:
                ex_bank = json.loads(await read_all(BANK / f"{level}.json"))
                if int(num) > len(ex_bank):
                    await refer_question.finish("题号错误，题库中没有这么多题目")
                else:
                    if ex_bank[num]["P"]:
                        await refer_question.send(MessageSegment.image(f"file:///{(IMG / ex_bank[num]['P']).absolute()}"))
                    await refer_question.send(f"{level} 类第 {num}题:\n{ex_bank[num]['Q']}\n：A、{ex_bank[num]['A']}\nB、{ex_bank[num]['B']}\nC、{ex_bank[num]['C']}\nD、{ex_bank[num]['D']}\n答案: 【A】")


# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/23 22:36
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : __init__.py.py
# @Software: PyCharm

from nonebot import on_command
from nonebot.adapters import Message
from nonebot.adapters.onebot.v11 import Bot, MessageEvent
from nonebot.params import CommandArg
from nonebot.typing import T_State

from .calculate_mgsl import getGrid
from ..utils import *

grid_ca = on_command("我的网格", block=True)


@grid_ca.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State):
    qq = event.user_id
    qth = await getQth(qq)
    if qth:
        await grid_ca.finish(f"你的网格是:{await getGrid(float(qth[0]), float(qth[1]))}")
    else:
        await grid_ca.finish(
            "你还没有设置QTH,请发送【绑定位置】\n查询具体位置的网格请发送【查询网格 经度 纬度】\n 经纬度用空格隔开")


grid_refer = on_command("查询网格", aliases={"计算网格"}, block=True)


@grid_refer.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State, args: Message = CommandArg()):
    qq = event.user_id
    qth = await getQth(qq)
    if args:
        args = str(args).strip().split(" ")
        if len(args) == 1 and isChinese(args[0]):
            user_input = args[0]
            location = await getLlByAd(user_input)
            if location:
                await grid_refer.finish(f"{user_input}的经纬度：{location[0], location[1]}\n"
                                        f"{'-' * 10}\n"
                                        f"网格是:{await getGrid(float(location[0]), float(location[1]))}\n"
                                        f"{'-' * 10}")
            else:
                await grid_refer.finish(f"未查询到{user_input}的经纬度")
        elif len(args) >= 2:
            await grid_refer.finish(f"{args[0], args[1]}网格是:\n{'-'*10}{await getGrid(float(args[0]), float(args[1]))}\n{'-'*10}")
        else:
            await grid_refer.finish("你的输入貌似有误，检查一下吧")
    else:
        if qth:
            await grid_refer.finish(
                f"你没有发送经纬度，为你计算你的网格\n{'-' * 10}\n你的网格：{await getGrid(float(qth[0]), float(qth[1]))}\n{'-' * 10}"
                f"\n计算指定位置请发送：\n【计算网格 北京】\n【查询网格 经度 纬度】\n (经纬度用空格隔开)")
        else:
            await grid_refer.finish("错误：请输入参数\n命令示例：\n【计算网格 北京】\n【计算网格 75.8656 (空格) 39.3809】")



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/21 2:15
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : log.py
# @Software: PyCharm
import os
import sys
import logging
from pathlib import Path
from .path import *


def log(path: str = LOG) -> logging.Logger:
    """
    日志
    :return:
    """
    Path.mkdir(LOCAL) if not Path.exists(LOCAL) else ...
    logger = logging.getLogger()
    logger.setLevel(logging.DEBUG)
    formatter = logging.Formatter('%(asctime)s-%(levelname)s: %(message)s')
    console_handler = logging.StreamHandler(sys.stdout)
    console_handler.formatter = formatter
    console_handler.setLevel(logging.INFO)
    logfile = path
    File_handler = logging.FileHandler(logfile, mode='a', encoding='utf-8')
    File_handler.setFormatter(formatter)
    File_handler.setLevel(logging.DEBUG)
    if not logger.handlers:
        logger.addHandler(File_handler)
        logger.addHandler(console_handler)
        print(logger.handlers)
    return logger



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/21 2:09
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : utils.py
# @Software: PyCharm
import json
import random
from typing import Union, Optional, TextIO

import httpx
import yaml
from nonebot.adapters.onebot.v11 import MessageSegment
from nonebot.matcher import Matcher

from .log import log
from .path import *

logger = log()


async def http_get(url: str, type_: str = "text") -> Union[str, dict]:
    """
    获取链接中的内容
    :param type_: text 或 json
    :param url: 链接
    :return:
    """
    try:
        async with httpx.AsyncClient() as client:
            result = (await client.get(url))
            if type_ == "json":
                return result.json()
            else:
                return result.text
    except Exception as e:
        logger.error(e)
        return ""


async def http_post(url: str, data: dict) -> str:
    """
    发送post请求
    :param url: 链接
    :param data: 参数
    :return:
    """
    try:
        async with httpx.AsyncClient() as client:
            return (await client.post(url, data=data)).text
    except Exception as e:
        logger.error(e)
        return ""


async def write_(path: str, data: str) -> None:
    """
    写入文件
    :param path: 文件路径
    :param data: 数据
    :return:
    """
    with open(path, 'w', encoding='utf-8') as f:
        f.write(data)
        f.close()


async def read_(path: Union[Path, str]) -> TextIO:
    """
    读取文件
    :param path: 文件路径
    :return:
    """
    with open(path, 'r', encoding='utf-8') as f:
        return f


async def read_all(path: Union[Path, str]) -> str:
    with open(path, 'r', encoding='utf-8') as f:
        return f.read()


async def yaml_load(path: str) -> dict:
    """
    读取yaml文件
    :param path: 文件路径
    :return:
    """
    with open(path, 'r', encoding='utf-8') as f:
        return yaml.load(f, Loader=yaml.FullLoader)


async def yaml_dump(path: str, data: dict) -> None:
    """
    写入yaml文件
    :param path: 文件路径
    :param data: 数据
    :return:
    """
    with open(path, 'w', encoding='utf-8') as f:
        yaml.dump(data, f, allow_unicode=True)
        f.close()


async def yaml_upload(path: str, data: dict) -> None:
    """
    更新yaml文件
    :param path: 文件路径
    :param data: 数据
    :return:
    """
    raw = await yaml_load(path)
    raw.update(data)
    await yaml_dump(path, raw)


def az2direction(azimuth: Union[float, int]):
    """
    将方位角转换为方位
    :param azimuth: 方位角
    :return: 方位
    """
    if 0 <= azimuth < 45:
        return "北偏东"
    elif 45 <= azimuth < 90:
        return "东偏北"
    elif 90 <= azimuth < 135:
        return "东偏南"
    elif 135 <= azimuth < 180:
        return "南偏东"
    elif 180 <= azimuth < 225:
        return "南偏西"
    elif 225 <= azimuth < 270:
        return "西偏南"
    elif 270 <= azimuth < 315:
        return "西偏北"
    elif 315 <= azimuth < 360:
        return "北偏西"


def isChinese(word):
    for ch in word:
        if '\u4e00' <= ch <= '\u9fff':
            return True
    return False


async def getQth(qq: int) -> Optional[list]:
    """
    获取QTH
    :param qq: QQ号
    :return: None 或 [经度， 纬度， 海拔]
    """
    qth_dict = await yaml_load(QTH)
    if qq in qth_dict:
        return qth_dict[qq]
    else:
        return None


async def getLlByAd(address: str, type_: str = "float") -> Optional[list]:
    """
    根据地名获取经纬度
    :param type_: float 或 str
    :param address: 地名
    :return: None 或 [经度， 纬度]
    """
    temp_dic: dict = await http_get(f"https://apis.map.qq.com/jsapi?qt=geoc&addr={address}", type_="json")
    if temp_dic and temp_dic["info"]["error"] == 0:
        if type_ == "float":
            return [float(temp_dic["detail"]["pointx"]), float(temp_dic["detail"]["pointy"])]
        elif type_ == "str":
            return [temp_dic["detail"]["pointx"], temp_dic["detail"]["pointy"]]
    else:
        return None


async def send_ex(
        matcher: Matcher,
        ex_bank: dict,
        last: int,
        qq: int,
        level: str,
        state: dict,
        img: Path,
        mode: str = "exec"
):
    """
    发送题目
    :param img: Path
    :param matcher: Matcher
    :param ex_bank: 题录字典
    :param last: 发送的题目
    :param qq: qq
    :param level: 等级
    :param state: state
    :param mode: exec 或 exam
    :return:
    """
    state_for_time = EXAM_CACHE / matcher.state["level"].upper() / f"{matcher.state['qq']}_state.txt"
    logger.info(f"发送题目：{last}")
    if mode == "exec":
        temp = {qq: {"level": level, "last": last}}
        await yaml_upload(EXERCISE_TEMP, temp)
    ex_list = state["ex_list"]
    IMG = img
    last = str(last)
    this_ex = ex_bank[last]
    pic = this_ex["P"]
    if pic:
        state["pic_to_send"] = pic
    else:
        state["pic_to_send"] = None
    this_answer = this_ex["A"]
    state["this_answer"] = this_answer
    this_answer_group = []
    for i in this_ex:
        if i in ["A", "B", "C", "D"]:
            this_answer_group.append(this_ex[i])
    this_answer_group = random.sample(this_answer_group, len(this_answer_group))
    dic_to_send = {"Q": this_ex["Q"], "A": this_answer_group[0], "B": this_answer_group[1], "C": this_answer_group[2],
                   "D": this_answer_group[3]}
    state['this_send'] = dic_to_send
    for i in dic_to_send:
        if dic_to_send[i] == this_answer:
            answer = i
            state["this_answer"] = answer
    reply = (
        f"{last if not ex_list else ex_list.index(int(last)) + 1}、{dic_to_send['Q']}\n"
        f"A. {dic_to_send['A']}\n"
        f"B. {dic_to_send['B']}\n"
        f"C. {dic_to_send['C']}\n"
        f"D. {dic_to_send['D']}\n"
    )
    state_2 = {"this_config": state["this_config"], "correct": state["correct"], "wrong": state["wrong"]}

    await write_(state_for_time, json.dumps(state_2))
    if state["pic_to_send"]:
        await matcher.send(MessageSegment.image(f"file:///{(IMG / state['pic_to_send']).absolute()}"))
        await matcher.reject(state["user_notice"] + reply)
    else:
        await matcher.reject(state["user_notice"] + reply)
    return state



import io
import sys

import setuptools

with open("README.md", "r", encoding="utf-8", errors="ignore") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nonebot-plugin-cqsat",
    version="0.0.8",
    author="yzyyz1387",
    author_email="youzyyz1384@gmail.com",
    keywords=("ham", "nonebot2", "nonebot", "radio", "nonebot_plugin"),
    description="""nonebot2 业余无线电卫星""",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/yzyyz1387/cqsat",
    packages=setuptools.find_packages(include=["cqsat", "cqsat.*"]),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)",
        "Operating System :: OS Independent",
    ],
    include_package_data=True,

    platforms="any",
    install_requires=["httpx", "pyephem", "python_dateutil", "PyYAML",
                      'nonebot-adapter-onebot>=2.0.0-beta.1', 'nonebot2>=2.0.0-beta.4'
                      ]
)



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/24 0:20
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : sat_handle.py
# @Software: PyCharm
import json
from datetime import datetime, timedelta

import nonebot
from dateutil.parser import ParserError
from dateutil.parser import parse
from nonebot import on_command, require
from nonebot.adapters import Message
from nonebot.adapters.onebot.v11 import Bot, GroupMessageEvent, MessageSegment, MessageEvent
from nonebot.adapters.onebot.exception import ActionFailed
from nonebot.internal.params import ArgStr
from nonebot.params import CommandArg
from nonebot.typing import T_State

from .calculate_sat import download_ham_sat, data2Tle, calculate, get_tian_gong
from ..log import log
from ..path import *
from ..utils import *

logger = log()
Path.mkdir(LOCAL) if not Path.exists(LOCAL) else ...

bind_qth = on_command("绑定位置", aliases={"绑定QTH", "绑Qth", "绑定qth"}, block=True)


@bind_qth.got("QTH", prompt="请输入：\n  地名 \n或者输入：\n   经度 纬度 海拔\n\n参数用空格分隔\n")
async def _(
        event: MessageEvent,
        state: T_State,
        qth: str = ArgStr("QTH")):
    if not Path.exists(LOCAL):
        Path.mkdir(LOCAL)
    qq = event.user_id
    if qth in ["取消", "算了"]:
        await bind_qth.finish("已取消操作...")
    qth_list = str(qth).split(" ")
    logger.info(len(qth_list))
    if len(qth_list) == 1:
        temp_list = await getLlByAd(qth_list[0], type_="str")
        if temp_list:
            qth_list = temp_list
            qth_list.append(1000)
            await bind_qth.send(f"已自动获取到{qth}的经纬度信息, 海拔默认为1000米")
        else:
            await bind_qth.finish(f"没有找到{qth}的经纬度，请重新输入...")
    elif len(qth_list) < 3:
        await bind_qth.reject_arg("QTH",
                                  "你的输入貌似有错误呢，请输入：\n  地名 \n或者输入：\n   经度 纬度 海拔\n\n参数用空格分隔\n")
    # else:
    logger.info(qth_list)
    try:
        if float(qth_list[0]) and float(qth_list[1]) and float(qth_list[2]):
            state["qth"] = qth_list
            qth_data = {qq: qth_list}
            if not Path.exists(QTH):
                await yaml_dump(QTH, qth_data)
            else:
                await yaml_upload(QTH, qth_data)
            await bind_qth.finish(f"绑定成功：\n{qth_list}")
    except ValueError:
        await sub.reject_arg("QTH", "请输入：\n  地名 \n或者输入：\n   经度 纬度 海拔\n\n参数用空格分隔\n")


sub = on_command("订阅卫星", priority=2, block=True)


@sub.got("Sat", prompt="你要订阅那颗卫星？\n多颗卫星用空格分隔")
async def _(
        event: GroupMessageEvent,
        state: T_State,
        sat: str = ArgStr("Sat"), ):
    qq = event.user_id
    group = event.group_id
    sat_s = (await data2Tle())
    if sat in ["取消", "算了"]:
        await sub.finish("已取消操作...")
    try:
        qth = (await yaml_load(QTH))
    except FileNotFoundError:
        qth = {}
        await sub.finish("没有找到QTH文件，请发送【绑定位置】来绑定QTH\n绑定后可以发送【订阅卫星】来订阅卫星")
    if qq in qth:
        if sat == "天宫" or sat == "中国空间站" or sat in ["TIANGONG", "tiangong"]:
            state["sat"] = ["天宫"]
        else:
            user_sat = sat.split(" ")
            if len(user_sat) == 1:
                if sat.upper() in sat_s:
                    state["sat"] = [sat.upper()]
                else:
                    await sub.reject_arg("Sat", f"没有找到{sat},请重新输入：")
            else:
                temp = []
                for sat_ in user_sat:
                    if sat_ == "天宫" or sat == "中国空间站" or sat in ["TIANGONG", "tiangong"]:
                        temp.append("天宫")
                    elif sat_.upper() in sat_s:
                        temp.append(sat_.upper())
                    else:
                        await sub.send(f"没有找到{sat_}")
                state["sat"] = temp
    else:
        await sub.finish("请先发送【绑定位置】来绑定QTH\n绑定后可以发送【订阅卫星】来订阅卫星")


@sub.got("E_angle", prompt="请输入最低仰角：")
async def _(
        event: GroupMessageEvent,
        state: T_State,
        ang: str = ArgStr("E_angle")):
    qq = event.user_id
    group = event.group_id
    # sat = state["sat"]
    if ang in ["取消", "算了"]:
        await sub.finish("已取消操作...")
    try:
        qth = (await yaml_load(QTH))
    except FileNotFoundError:
        qth = {}
        await sub.finish("没有找到QTH文件，请发送【绑定位置】来绑定QTH\n绑定后可以发送【订阅卫星】来订阅卫星")
    try:
        ang = ang.replace("°", "").replace("度", "")
        if not 0 < int(ang) <= 90:
            await sub.reject_arg("E_angle", "仰角输入有误，应该在0-90之间，请重新输入：")
        else:
            state["ang"] = ang
        if not Path.exists(CONFIG):
            data = {group: {qq: {}}}
            for sat in state["sat"]:
                temp = {"仰角": state['ang'], "qth": qth[qq], "last_send": ""}
                data[group][qq][sat] = temp
            await yaml_dump(CONFIG, data)
        else:
            data = (await yaml_load(CONFIG))
            for sat in state["sat"]:
                if qq in qth:
                    if group in data:
                        if qq in data[group]:
                            data[group][qq][sat] = {"仰角": state['ang'], "qth": qth[qq], "last_send": ""}
                        else:
                            data[group][qq] = {sat: {"仰角": state['ang'], "qth": qth[qq], "last_send": ""}}
                    else:
                        data[group] = {qq: {sat: {"仰角": state['ang'], "qth": qth[qq], "last_send": ""}}}
                    await yaml_dump(CONFIG, data)
        await sub.finish("订阅成功:"
                         f"\n卫星：{state['sat']}"
                         f"\n最低仰角：{state['ang']}"
                         f"\nQTH：{qth[qq]}")
    except ValueError:
        await sub.reject_arg("E_angle", "仰角输入有误，应该是0-90之间的数字，请重新输入：")


unsub = on_command("取消订阅", block=True)


@unsub.handle()
async def unsub_(bot: Bot, event: GroupMessageEvent, state: T_State, args: Message = CommandArg()):
    qq = event.user_id
    group = event.group_id
    if args:
        sat_s = str(args).split(" ")
        data = (await yaml_load(CONFIG))
        for sat in sat_s:
            if isChinese(sat):
                pass
            else:
                sat = sat.upper()
            if group in data:
                if qq in data[group]:
                    if sat in data[group][qq]:
                        del data[group][qq][sat]
                        await yaml_dump(CONFIG, data)
                        await sub.send(f"已取消订阅{sat}")
                    else:
                        await unsub.finish(f"你在本群没有订阅{sat}")
                else:
                    await unsub.finish("你在本群没有订阅卫星")
            else:
                await unsub.finish("本群没有用户订阅卫星\n订阅请发送【订阅卫星】")
    else:
        await unsub.finish("取消订阅命令为：\n   取消订阅+卫星名称\n例如：\n   取消订阅AO-72\n多颗卫星用空格分隔")


refer_sat_subs = on_command("查询订阅", aliases={"我的订阅"}, block=True)


@refer_sat_subs.handle()
async def refer_sat_(bot: Bot, event: GroupMessageEvent, state: T_State, args: Message = CommandArg()):
    qq = event.user_id
    group = event.group_id
    data = (await yaml_load(CONFIG))
    if group in data:
        if qq in data[group]:
            await refer_sat_subs.send(MessageSegment.at(qq) + ",你在本群订阅了以下卫星：\n")
            reply = ""
            for sat in data[group][qq]:
                reply += sat + "\n"
            await refer_sat_subs.send(reply)
        else:
            await refer_sat_subs.send(f"{qq}在本群没有订阅卫星")
    else:
        await refer_sat_subs.send(f"本群没有用户订阅卫星\n订阅请发送【订阅卫星】")


refer_sat = on_command("卫星列表", block=True)


@refer_sat.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State, args: Message = CommandArg()):
    sat_dict = (await data2Tle())
    sat_list = []
    for i in sat_dict.keys():
        sat_list.append(i)
    reply = "\n".join(sat_list)
    await refer_sat.send(f"卫星列表：\n{reply}")


specified = on_command("查询卫星", aliases={"计算卫星"}, block=True)


@specified.handle()
async def _(bot: Bot, event: MessageEvent, state: T_State, args: Message = CommandArg()):
    qq = event.user_id
    input_arg = str(args).split(" ")
    sat_dict = (await data2Tle())
    sat = input_arg[0]
    try:
        qth = (await yaml_load(QTH))[qq]
    except KeyError:
        qth = ["0", "0", "0"]
        await specified.finish("你没有设置QTH，请发送【绑定位置】绑定QTH")
    except FileNotFoundError:
        qth = ["0", "0", "0"]
        await specified.finish("你没有设置QTH，请发送【绑定位置】绑定QTH")
    try:
        add = int(input_arg[1])
    except IndexError:
        add = 0
        await specified.send(f"你没有发送时间，将计算 {sat} 现在的状态")
    time = (datetime.utcnow() + timedelta(minutes=add)).strftime("%Y-%m-%d %H:%M:%S")
    send_time = (datetime.now() + timedelta(minutes=add)).strftime("%Y-%m-%d %H:%M:%S")
    if not isChinese(sat):
        sat = sat.upper()
    else:
        sat_dict = (await get_tian_gong())
    if sat in sat_dict:
        out = await calculate(sat, qth, time=time)
        reply = f"对于QTH:：{qth}\n{sat} 在 {send_time} ：\n仰角为：{round(float(out[1]), 2)}°\n方位角:{round(float(out[0]), 2)}°\n相对速率为：{round(float(out[2]), 2)} "
        await specified.send(reply)
    else:
        await specified.send(f"{sat}不存在")


async def aps():
    if not Path.exists(CONFIG):
        return
    elif read_all(CONFIG) == "":
        return
    today = datetime.now().strftime("%Y-%m-%d")
    try:
        # FIXME 目前是每次都下载
        await download_ham_sat()
        data = (await yaml_load(CONFIG))
    except FileNotFoundError:
        data = await download_ham_sat()

    for group in data:
        for qq in data[group]:
            for sat in data[group][qq]:
                try:
                    last = parse(data[group][qq][sat]["last_send"])
                except ParserError:
                    last = parse("2001-11-04 00:00:00")
                except TypeError:
                    last = parse("2001-11-04 00:00:00")
                now_to_calculate = (datetime.now()).strftime("%Y-%m-%d %H:%M:%S")
                min_add = 10
                if int((parse(now_to_calculate) - last).total_seconds() / 60) >= 60:
                    now = (datetime.utcnow() + timedelta(minutes=min_add)).strftime("%Y-%m-%d %H:%M:%S")
                    result = await calculate(sat, data[group][qq][sat]["qth"], now)
                    sated = []
                    if float(result[1]) > 0:
                        logger.info(f"@{qq} 订阅的 {sat} 将入境，正准备计算最高仰角")
                        i = 0
                        az_list = []
                        alt_list = []
                        time_highest_point = {}
                        result_ = await calculate(sat, data[group][qq][sat]["qth"], now)
                        while float(result_[1]) >= 0:
                            now = (datetime.utcnow() + timedelta(minutes=min_add + int(i))).strftime(
                                "%Y-%m-%d %H:%M:%S")
                            result_ = await calculate(sat, data[group][qq][sat]["qth"], now)
                            i += 1
                            az_list.append(float(result_[0]))
                            alt_list.append(float(result_[1]))
                            logger.debug("方位角信息：")
                            logger.debug(az_list)
                            logger.debug("仰角信息：")
                            logger.debug(alt_list)
                            time_highest_point[now] = float(result_[1])
                        a1 = sorted(time_highest_point.items(), key=lambda x: x[1], reverse=True)
                        if float(a1[0][1]) >= float(data[group][qq][sat]["仰角"]):
                            logger.info("最高仰角大于用户设定的值，正准备提醒用户")
                            if sat not in sated:
                                logger.info("成功预测到卫星。正在尝试发送数据")
                                azimuth = float(result_[0])
                                direction = az2direction(azimuth)
                                azimuth_ = float(result[0])
                                r_direction = az2direction(azimuth_)
                                try:
                                    await nonebot.get_bot().send_group_msg(
                                        group_id=group,
                                        message=MessageSegment.at(qq) + f'\n'
                                                                        f'{sat} 卫星将于 {(parse(a1[0][0]) + timedelta(hours=8)).strftime("%Y-%m-%d %H:%M:%S")} 出现最高仰角\n'
                                                                        f'角度为{round(float(a1[0][1]), 1)}°\n'
                                                                        f'入境方位为：{r_direction}\n'
                                                                        f'入境方位角：{round(float(result[0]), 1)}\n出境方位为：{direction}\n '
                                                                        f'出境方位角：{round(azimuth, 1)}°')
                                except ActionFailed:
                                    logger.error("消息发送失败，账号可能被风控")
                                sated.append(sat)
                            data[group][qq][sat]["last_send"] = now_to_calculate
                            await yaml_dump(CONFIG, data)
                        else:
                            logger.info("最高仰角小于用户设定的值，不需要提醒")
                else:
                    logger.info("一小时内该卫星已过境，跳过此卫星")


scheduler = require("nonebot_plugin_apscheduler").scheduler

scheduler.add_job(aps, 'cron', minute="0/1", id="sat")



# python3
# -*- coding: utf-8 -*-
# @Time    : 2022/8/31 3:05
# @Author  : yzyyz
# @Email   :  youzyyz1384@qq.com
# @File    : __init__.py.py
# @Software: PyCharm
import json
import random

from nonebot import on_command
from nonebot.adapters.onebot.v11 import MessageSegment, MessageEvent, Event, GroupMessageEvent, PrivateMessageEvent
from nonebot.internal.params import ArgStr, ArgPlainText
from nonebot.typing import T_State

from ..utils import *
from . import exam

RESOURCE = Path(__file__).parent.parent / "cqsat_resource"
BANK = RESOURCE / "bank"
IMG = BANK / 'imgs'
do_exercise = on_command("刷题", block=True)


@do_exercise.got("level", prompt="请你要练习哪个等级的题目？（A B C）")
async def _(
        event: Event,
        state: T_State,
        answer=None,
        level: str = ArgStr("level")):
    if isinstance(event, GroupMessageEvent):
        qq = event.user_id
        state["user_notice"] = MessageSegment.at(qq)
    elif isinstance(event, PrivateMessageEvent):
        qq = event.user_id
        state["user_notice"] = ""
    if not Path.exists(EXERCISE_TEMP):
        await yaml_dump(EXERCISE_TEMP, {})
    if level in ["取消", "算了"]:
        await do_exercise.finish("已取消操作...")
    elif level.upper() not in ["A", "B", "C"]:
        await do_exercise.reject("输入错误，请重新输入...")
    elif level.upper() in ["A", "B", "C"]:
        state['level'] = level.upper()
        state['qq'] = qq
    already_done = (await yaml_load(EXERCISE_TEMP))
    if qq in already_done:
        last = already_done[qq]['last']
        last_level = already_done[qq]['level']
        if level.upper() == last_level:
            await do_exercise.send(state["user_notice"] + f"你上次练习{last_level}到了{last}题,将继续练习")
        else:
            await do_exercise.send(state["user_notice"] +
                                   f"你上次练习的是{last_level},进度为 【{last}】 题，该进度已删除\n正开始{level.upper()}类题目的练习")
            last = 1
            temp = {qq: {"level": level, "last": 1}}
            await yaml_upload(EXERCISE_TEMP, temp)
    else:
        last = 1
        temp = {qq: {"level": level, "last": 1}}
        await yaml_upload(EXERCISE_TEMP, temp)
    state['last'] = last
    state["is_send"] = False


@do_exercise.got("answer", prompt="回复任意内容开始...")
async def _(matcher: Matcher, state: T_State, reply: str = ArgPlainText("answer")):
    level = state["level"]
    qq = state["qq"]
    last = state['last']
    ex_bank = json.loads(await read_all(BANK / f"{level}.json"))
    if reply in ["取消", "算了", "退出", "不做了", "怎么退出"]:
        temp = {qq: {"level": level, "last": last}}
        await yaml_upload(EXERCISE_TEMP, temp)
        await do_exercise.finish("已取消操作...")
    if state["is_send"]:
        if reply.upper() in ["A", "B", "C", "D"]:
            for i in range(len(ex_bank) - last):
                # if state["this_send"][reply.upper()] != state["this_answer"]:
                if reply.upper() != state["this_answer"]:
                    await do_exercise.send("×")
                    state = (
                        await send_ex(
                            matcher,
                            ex_bank,
                            last,
                            qq,
                            level,
                            state,
                            img=IMG,
                            mode="exec"
                        )
                    )
                elif reply.upper() == state["this_answer"]:
                    last += 1
                    state["last"] = last
                    await do_exercise.send("√")
                    state = (
                        await send_ex(
                            matcher,
                            ex_bank,
                            last,
                            qq,
                            level,
                            state,
                            img=IMG,
                            mode="exec"
                        )
                    )
        else:
            await do_exercise.reject(state["user_notice"] + "输入错误，请重新输入...")
    elif not state["is_send"]:
        state["is_send"] = True
        state = (
            await send_ex(
                matcher,
                ex_bank,
                last,
                qq,
                level,
                state,
                img=IMG,
                mode="exec"
            )
        )
